@@ -17,19 +17,24 @@
 #ifndef GLOBUS_XIO_TCP_DRIVER_INCLUDE
 #define GLOBUS_XIO_TCP_DRIVER_INCLUDE
 /**
- * @file
- * Header file for XIO TCP Driver
+ * @file globus_xio_tcp_driver.h
+ * @brief Header file for XIO TCP Driver
  */
 #include "globus_xio_system.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
- * @defgroup tcp_driver Globus XIO TCP Driver
+ * @defgroup globus_xio_tcp_driver Globus XIO TCP Driver
+ * @ingroup globus_xio
  * The IPV4/6 TCP socket driver.
  */
  
 /**
- * @defgroup tcp_driver_instance Opening/Closing
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_instance Opening/Closing
+ * @ingroup globus_xio_tcp_driver
  * 
  * An XIO handle with the tcp driver can be created with either
  * @ref globus_xio_handle_create() or @ref globus_xio_server_register_accept().
@@ -56,8 +61,8 @@
  */
  
 /**
- * @defgroup tcp_driver_io Reading/Writing
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_io Reading/Writing
+ * @ingroup globus_xio_tcp_driver
  * 
  * Both the @ref globus_xio_register_read() and
  * @ref globus_xio_register_write() calls follow similar semantics as described
@@ -84,8 +89,8 @@
  */
  
 /**
- * @defgroup tcp_driver_server Server
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_server Server
+ * @ingroup globus_xio_tcp_driver
  * 
  * @ref globus_xio_server_create() causes a tcp listener socket to be created
  * and listened upon.  @ref globus_xio_server_register_accept() performs an
@@ -99,8 +104,8 @@
  */
  
 /**
- * @defgroup tcp_driver_envs Env Variables
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_envs Env Variables
+ * @ingroup globus_xio_tcp_driver
  * 
  * The tcp driver uses the following environment variables
  * - GLOBUS_HOSTNAME Used when setting the hostname in the contact string
@@ -130,8 +135,8 @@
  */
 
 /**
- * @defgroup tcp_driver_cntls Attributes and Cntls
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_cntls Attributes and Cntls
+ * @ingroup globus_xio_tcp_driver
  * 
  * Tcp driver specific attrs and cntls.
  * 
@@ -141,12 +146,12 @@
  * @see globus_xio_data_descriptor_cntl()
  */
 /**
- * @defgroup tcp_driver_types Types
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_types Types
+ * @ingroup globus_xio_tcp_driver
  */
 /**
- * @defgroup tcp_driver_errors Error Types
- * @ingroup tcp_driver
+ * @defgroup globus_xio_tcp_driver_errors Error Types
+ * @ingroup globus_xio_tcp_driver
  * 
  * The TCP driver is very close to the system code, so most errors
  * reported by it are converted from the system errno. A few of the exceptions
@@ -160,7 +165,7 @@
 
 /**
  * Invalid handle type
- * @ingroup tcp_driver_types
+ * @ingroup globus_xio_tcp_driver_types
  * @hideinitializer
  * @see GLOBUS_XIO_TCP_SET_HANDLE
  */
@@ -168,7 +173,7 @@
 
 /**
  * TCP driver specific error types
- * @ingroup tcp_driver_errors
+ * @ingroup globus_xio_tcp_driver_errors
  */
 typedef enum
 {
@@ -192,13 +197,13 @@ typedef enum
 
 /**
  * TCP driver specific cntls
- * @ingroup tcp_driver_cntls
+ * @ingroup globus_xio_tcp_driver_cntls
  */
 typedef enum
 {
     /** GlobusVarArgEnum(attr)
      * Set the tcp service name to bind to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create().
      * 
      * @param service_name
@@ -206,14 +211,14 @@ typedef enum
      *      service name cannot be resolved, the port (if one is set) will
      *      be used instead.
      *
-     * string opt: port=<string>
+     * string opt: port=<em>string</em>
      */
     /* const char *                     service_name */
     GLOBUS_XIO_TCP_SET_SERVICE,
     
     /** GlobusVarArgEnum(attr)
      * Get the tcp service name to bind to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param service_name_out
      *      A pointer to the service name will be stored here  If none is set,
@@ -225,7 +230,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the tcp port number to bind to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create(). The default
      * port number is 0 (system assigned)
      * 
@@ -234,14 +239,14 @@ typedef enum
      *      service name is also set, this will only be used if that can't be
      *      resolved.
      *
-     * string opt: port=<int>
+     * string opt: port=<em>int</em>
      */
     /* int                              listener_port */
     GLOBUS_XIO_TCP_SET_PORT,
     
     /** GlobusVarArgEnum(attr)
      * Get the tcp port number to bind to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param listener_port_out
      *      The port will be stored here.
@@ -251,7 +256,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the listener backlog on a server.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create(). The default
      * backlog is -1 (system maximum)
      * 
@@ -266,7 +271,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Get the listener backlog on an attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param listener_backlog_out
      *      The backlog will be stored here.
@@ -276,7 +281,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the tcp port range to confine the server to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() where no 
      * specific service or port has been set.  It overrides the range set in
      * the GLOBUS_TCP_PORT_RANGE env variable.  If 'restrict port' is true,
@@ -290,7 +295,7 @@ typedef enum
      * 
      * @see GLOBUS_XIO_TCP_SET_RESTRICT_PORT
      *
-     * string opt: listen_range=<int>,<int>
+     * string opt: listen_range=<em>int</em>,<em>int</em>
      */
     /* int                              listener_min_port,
      * int                              listener_max_port */
@@ -298,7 +303,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Get the tcp port range on an attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param listener_min_port_out
      *      The lower bound will be stored here.
@@ -312,7 +317,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle, server)
      * Get the tcp socket handle on an attr, handle, or server.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param handle_out
      *      The tcp socket will be stored here. If none is set,
@@ -323,7 +328,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the tcp socket to use for a handle or server.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() or
      * @ref globus_xio_register_open().
      * 
@@ -336,21 +341,21 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the interface to bind a listener or connection to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() or
      * @ref globus_xio_register_open.
      * 
      * @param interface
      *      The interface to use.  Can be a hostname or numeric IP
      *
-     * string opt: iface=<string>
+     * string opt: iface=<em>string</em>
      */
     /* const char *                     interface */
     GLOBUS_XIO_TCP_SET_INTERFACE,
     
     /** GlobusVarArgEnum(attr)
      * Get the interface on the attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param interface_out
      *      A pointer to the interface will be stored here  If one is set,
@@ -362,7 +367,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Enable or disable the listener or connector range constraints.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() or
      * @ref globus_xio_register_open().  This enables or ignores the port range
      * found in the attr or in then env.  By default, those ranges are enabled.
@@ -378,7 +383,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Get the restrict port flag.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param restrict_port_out
      *      The restrict port flag will be stored here.
@@ -388,7 +393,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Reuse addr when binding.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() or
      * @ref globus_xio_register_open() to determine whether or not to allow
      * reuse of addresses when binding a socket to a port number.
@@ -396,14 +401,14 @@ typedef enum
      * @param resuseaddr
      *      GLOBUS_TRUE to allow, GLOBUS_FALSE to disallow (default)
      *
-     * string opt: reuse=<bool>
+     * string opt: reuse=<em>bool</em>
      */
     /* globus_bool_t                    resuseaddr */
     GLOBUS_XIO_TCP_SET_REUSEADDR,
     
     /** GlobusVarArgEnum(attr)
      * Get the reuseaddr flag on an attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param resuseaddr_out
      *      The reuseaddr flag will be stored here.
@@ -413,7 +418,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Restrict to IPV4 only.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_server_create() or
      * @ref globus_xio_register_open().  Disallow IPV6 sockets from being used
      * (default is to use either ipv4 or ipv6)
@@ -421,14 +426,14 @@ typedef enum
      * @param no_ipv6
      *      GLOBUS_TRUE to disallow ipv6, GLOBUS_FALSE to allow (default)
      *
-     * string opt: noipv6=<bool>
+     * string opt: noipv6=<em>bool</em>
      */
     /* globus_bool_t                    no_ipv6 */
     GLOBUS_XIO_TCP_SET_NO_IPV6,
     
     /** GlobusVarArgEnum(attr)
      * Get the no ipv6 flag on an attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param no_ipv6_out
      *      The no ipv6 flag will be stored here.
@@ -438,7 +443,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Set the tcp port range to confine the server to.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only on attrs for @ref globus_xio_register_open(). It overrides
      * the range set in the GLOBUS_TCP_SOURCE_RANGE env variable. If 
      * 'restrict port' is true, the connecting socket's local port will be
@@ -458,7 +463,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr)
      * Get the tcp source port range on an attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param connector_min_port_out
      *      The lower bound will be stored here.
@@ -472,7 +477,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Enable tcp keepalive.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to determine whether or not to periodically send "keepalive" messages
@@ -482,14 +487,14 @@ typedef enum
      * @param keepalive
      *      GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default)
      *
-     * string opt: keepalive=<bool>
+     * string opt: keepalive=<em>bool</em>
      */
     /* globus_bool_t                    keepalive */
     GLOBUS_XIO_TCP_SET_KEEPALIVE,
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the tcp keepalive flag.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param keepalive_out
      *      The tcp keepalive flag will be stored here.
@@ -499,7 +504,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Set tcp linger.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to determine what to do when data is in the socket's buffer when the
@@ -523,7 +528,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the tcp linger flag and time.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param linger_out
      *      The linger flag will be stored here.
@@ -537,7 +542,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Receive out of band data (tcp urgent data) in normal stream.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to choose whether out-of-band data is received in the normal data
@@ -551,7 +556,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the oobinline flag.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param oobinline_out
      *      The oobinline flag will be stored here.
@@ -561,7 +566,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Set the tcp socket send buffer size.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to set the size of the send buffer used on the socket.
@@ -569,14 +574,14 @@ typedef enum
      * @param sndbuf
      *      The send buffer size in bytes to use. (default is system specific)
      *
-     * string opt: sndbuf=<formatted int>
+     * string opt: sndbuf=<em>formatted int</em>
      */
     /* int                              sndbuf */
     GLOBUS_XIO_TCP_SET_SNDBUF,
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the tcp send buffer size on the attr or handle.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param sndbuf_out
      *      The send buffer size will be stored here.
@@ -586,7 +591,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Set the tcp socket receive buffer size.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to set the size of the receive buffer used on the socket. The receive
@@ -596,14 +601,14 @@ typedef enum
      * @param rcvbuf
      *      The receive buffer size in bytes. (default is system specific)
      *
-     * string opt: rcvbuf=<formatted int>
+     * string opt: rcvbuf=<em>formatted int</em>
      */
     /* int                              rcvbuf */
     GLOBUS_XIO_TCP_SET_RCVBUF,
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the tcp receive buffer size on the attr or handle.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param rcvbuf_out
      *      The receive buffer size will be stored here.
@@ -613,7 +618,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Disable Nagle's algorithm.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used on attrs for @ref globus_xio_server_create(), 
      * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
      * to determine whether or not to disable Nagle's algorithm. If set to 
@@ -623,14 +628,14 @@ typedef enum
      * @param nodelay
      *      GLOBUS_TRUE to disable nagle, GLOBUS_FALSE to enable (default)
      *
-     * string opt: nodelay=<bool>
+     * string opt: nodelay=<em>bool</em>
      */
     /* globus_bool_t                    nodelay */
     GLOBUS_XIO_TCP_SET_NODELAY,
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the tcp nodelay flag.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param nodelay_out
      *      The no delay flag will be stored here.
@@ -640,7 +645,7 @@ typedef enum
     
     /**GlobusVarArgEnum(dd)
      * Set tcp send flags.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * Used only for data descriptors to write calls.
      * 
      * @param send_flags
@@ -653,7 +658,7 @@ typedef enum
     
     /**GlobusVarArgEnum(dd)
      * Get tcp send flags.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param send_flags_out
      *      The flags to use will be stored here.
@@ -663,7 +668,7 @@ typedef enum
     
     /**GlobusVarArgEnum(handle, server)
      * Get local socket info.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param contact_string_out
      *      A pointer to a contact string for the local end of a connected
@@ -678,7 +683,7 @@ typedef enum
     
     /**GlobusVarArgEnum(handle, server)
      * Get local socket info.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param contact_string_out
      *      A pointer to a contact string for the local end of a connected
@@ -692,7 +697,7 @@ typedef enum
     
     /**GlobusVarArgEnum(handle)
      * Get remote socket info.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param contact_string_out
      *      A pointer to a contact string for the remote end of a connected
@@ -706,7 +711,7 @@ typedef enum
     
     /**GlobusVarArgEnum(handle)
      * Get remote socket info.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param contact_string_out
      *      A pointer to a contact string for the remote end of a connected
@@ -720,7 +725,7 @@ typedef enum
     
     /**GlobusVarArgEnum(attr)
      * Change the default attr values.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param affect_global
      *      If GLOBUS_TRUE, any future cntls on this attr will access
@@ -738,7 +743,7 @@ typedef enum
     /** GlobusVarArgEnum(attr, handle)
      * Enable true blocking io when making globus_xio_read/write() calls.
      * Note: use with caution.  you can deadlock an entire app with this.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param use_blocking_io
      *      If GLOBUS_TRUE, true blocking io will be enabled.
@@ -749,7 +754,7 @@ typedef enum
     
     /** GlobusVarArgEnum(attr, handle)
      * Get the blocking io status in use or in attr.
-     * @ingroup tcp_driver_cntls
+     * @ingroup globus_xio_tcp_driver_cntls
      * 
      * @param use_blocking_io_out
      *      The flag will be set here.  GLOBUS_TRUE for enabled.
@@ -762,7 +767,7 @@ typedef enum
 
 /**
  * TCP driver specific types
- * @ingroup tcp_driver_types
+ * @ingroup globus_xio_tcp_driver_types
  * @hideinitializer
  */
 typedef enum
@@ -774,4 +779,8 @@ typedef enum
     GLOBUS_XIO_TCP_SEND_OOB = MSG_OOB
 } globus_xio_tcp_send_flags_t;
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
