@@ -70,218 +70,203 @@ do                                                                          \
  * @ingroup globus_xio
  *
  * Globus XIO introduces a notion of a driver stack to its API.
- * With in globus_xio every IO operation must occur on a globus_xio 
+ * Within globus_xio every I/O operation must occur on a globus_xio 
  * handle.  Associated with each handle is a stack of drivers.
- * A driver is a module piece of code that implements the globus_xio
+ * A driver is a modulular piece of code that implements the globus_xio
  * driver interface.  The purpose of a driver is manipulate data passed
  * in by the user in someway.  Each driver in a stack will serve its own 
  * unique purpose.
  * 
- * IO operations pass from driver to driver, starting at the top of the 
+ * I/O operations pass from driver to driver, starting at the top of the 
  * stack and ending at the bottom.  When the bottom layer driver finishes 
  * with the operation it signals globus_xio that it has completed.  
- * Completion notification then follows the driver stack up to the top.
+ * Completion notification then flows up to the top of the driver stack.
  *
  *  @section driver_types Driver Types
  * 
  *  <dl>
- *     <dt><tt>Transport driver:</tt></dt>
+ *     <dt>Transport driver:</dt>
  *     <dd>
- *     A transport driver is one that is responsible for actually putting
- *     bytes onto the wire.  For example: A TCP driver or a UDP driver would
- *     be an example of transport drivers.  
+ *     A transport driver is one that is responsible for
+ *     data communication. For example, a TCP or UDP driver would
+ *     transmit data via network sockets, or a file driver would write
+ *     data to a file.
  *     
- *     Per driver stack there must be exactly one transport driver and
- *     must be at the bottom of the stack.  A transform driver is defined
- *     by its lack of passing an operation to the next driver in the stack.
- *     This type of driver does not rely on globus_xio for further completion
- *     of an operation, rather it is self sufficient in this task.
+ *     There must be exactly one transport driver in a stack, at its bottom.
+ *     A transport driver never passes an operation to another driver in
+ *     the stack. Instead, this type of driver relies on globus_xio system
+ *     functions to implement data operations.
  *     </dd>
  *
- *     <dt><tt>Transform driver:</tt></dt>
+ *     <dt>Transform driver:</dt>
  *     <dd>
- *     A transform driver is any intermediate driver in the stack.  These 
- *     drivers are identified by their reliance on the driver stack to
- *     complete the operation.  These drivers must pass the operation
- *     down the stack because they cannot complete it themselves.  An
- *     example of a transform driver would be a gsi driver.  This driver
- *     would wrap and unwrap messages, but would not be able to complete
- *     the transport itself, so it would rely on the remaining drivers in
- *     the stack.
+ *     A transform driver is any intermediate driver in the stack. A transform 
+ *     driver relies on some other drivers on the driver stack to
+ *     perform the data transport operation.  An example of a transform driver
+ *     would one which implements a network protocol such as http.  This driver
+ *     would frame and parse messages, but would rely on other drivers in
+ *     the XIO stack.
+ *
+ *     This allows additional transforms to happen in the XIO stack, such as
+ *     layering the http protocol on top of an SSL protocol driver before
+ *     transmitting the data via TCP.
  *     </dd>
  *  </dl>
  *
  *  @section driver_api Driver API 
  *  The Globus XIO Driver API is a set of functions and interfaces
- *  to allow a developer to create a back-end driver for globus_xio.
+ *  to allow a developer to create an XIO driver.
  *  To create a driver the user must implement all of the interface
  *  functions in the driver specification.
- *  There are also a set of functions provide to assist the driver
- *  author in implementation.
+ *  There is also a set of functions provided to assist the driver
+ *  author in implementing a driver.
  *
- *    @subsection driver_api_quickstart Quick Start
- *    For basic driver needs, the user will have to pay attention to a
- *    few new structures and concepts.
+ *  @subsection driver_api_quickstart Quick Start
+ *  For basic driver needs, the developer will have to pay attention to a
+ *  few structures and concepts.
  *
- *    <dl>
- *        <dt><tt>globus_xio_operation_t</tt></dt>
+ *  <dl>
+ *        <dt><code>globus_xio_operation_t</code></dt>
  *        <dd>
  *        This structure represents a request for an operation.  If
  *        the driver can service the operation it does so and the
- *        calls the appropriate finish_operation() function.  If the
- *        driver cannot completely service the operation it can pass()
- *        it along to the next driver in the stack.  As soon as the
+ *        calls the appropriate <em>finished</em> function.  If the
+ *        driver cannot completely service the operation it can <em>pass</em>
+ *        it to the next driver in the stack. As soon as the
  *        operation structure is either finished or passed it is no 
  *        longer valid for use in any other function.
  *        </dd>
  *
- *        <dt><tt>globus_xio_driver_handle_t</tt></dt>
+ *        <dt><code>globus_xio_driver_handle_t</code></dt>
  *        <dd>
- *        A driver_handle represents a open handle to the driver stack for xio.
- *        The driver obtains a driver_handle by calling 
- *        globus_xio_driver_open().
- *        When the open operation completes (it callback is called) the 
- *        driver then has a driver_handle.  The driver_handle 
- *        allows the user to do some
+ *        A globus_xio_driver_handle_t represents an open handle to the driver
+ *        stack for XIO. The driver obtains a driver_handle by calling 
+ *        globus_xio_operation_get_driver_handle().
+ *        The driver_handle allows the user to do some
  *        complex things that will be described later.
  *        </dd>
- *
- *        <dt><tt>globus_xio_stack_t</tt></dt>
- *        <dd>
- *        This structure provides the driver with information about the
- *        driver stack  It is mainly used for creating driver_handle as a
- *        parameter to globus_xio_driver_open().
- *        </dd>
  *    </dl>
  * 
- *  \par Typical Sequence:
+ *  @subsection globus_xio_driver_typical Typical Sequence:
  *  Here is a typical sequence of events for a globus_xio transform
- *  driver:
+ *  driver. All operations are initiated either by the application calling
+ *  a function in the Globus XIO API, or from a driver above the transform
+ *  driver in the stack passing an operation to this driver.
  *
  *  <dl>
- *           <dt><tt>Open</tt></dt>
+ *           <dt>Open</dt>
  *           <dd>
- *           globus_xio_driver_open_t is called.  The user calls
- *           globus_xio_driver_open() passing it the operation and 
- *           the stack and a callback.  When the open callback is called 
- *           the driver is given a new operation as a parameter.  The driver
- *           will then call globus_xio_driver_finished_open() passing
- *           it the now initialized driver_handle and the newly received
- *           operation.
+ *           XIO calls the globus_xio_driver_transform_open_t function of the
+ *           driver, passing it the operation and, in the case of a passive
+ *           open, the link from a server accept. The driver typically
+ *           allocates a private data structure containing the state it wishes
+ *           to associate with this handle, and passes this along with a
+ *           callback function pointer to globus_xio_driver_pass_open(). 
+ *           This allows the other drivers in the XIO stack to continue to
+ *           process the operation.
  *
- *           The call to globus_xio_driver_finished_open() does two things:
- *           <ol>
- *           <li>it tells globus_xio that this driver has finished its open
- *           operation</li>
- *           <li>it gives xio the driver_handle (which contains
- *           information on the drivers below it).</li>
- *           </ol>
+ *           After the lower drivers in the stack have completed processing
+ *           the open, they will call globus_xio_driver_finished_open(),
+ *           which will in turn call the callback function which the
+ *           driver passed as a parameter to globus_xio_driver_pass_open().
+ *           The driver then does any post-open operation needed, and then
+ *           finishes the operation by calling
+ *           globus_xio_driver_finished_open(), passing the private state
+ *           data as the <em>driver_handle</em> parameter.  This state
+ *           data will be passed to future operation implementations done 
+ *           with this handle.
+ *
+ *           <dt>Read/Write</dt>
+ *           <dd>XIO calls the driver's
+ *           globus_xio_driver_read_t or globus_xio_driver_write_t function
+ *           with an operation as a parameter and the state date from
+ *           the globus_xio_driver_finished_open() call. The driver then 
+ *           transforms the data if necessary and then calls
+ *           the globus_xio_driver_pass_read() or
+ *           globus_xio_driver_pass_write()
+ *           which passes the operation down the XIO driver stack. When
+ *           the driver below it on the XIO stack calls
+ *           globus_xio_driver_finished_read() or
+ *           globus_xio_driver_finished_write(), the callback which was
+ *           passed is invoked. In that callback, the driver should itself
+ *           call globus_xio_driver_finished_read() or
+ *           globus_xio_driver_finished_write() to indicate completion.
  *           </dd>
  *
- *           <dt><tt>Read/Write</tt></dt>
- *           <dd>The read or write interface function is called.  It receives
- *           a operation as a parameter.  The driver then calls the 
- *           appropriate pass operation and waits for the callback.  When
- *           the callback is received the driver calls finished_operation
- *           passing in the operation structure it received in the callback
- *           </dd>
- *
- *           <dt><tt>Close</tt></dt>
- *           <dd>
- *            The close interface function is called and is passed an
- *            operation and a driver_handle.  The driver will call 
- *            globus_xio_driver_close() passing it the operation.
+ *           <dt>Close</dt>
+ *           <dd>XIO calls the close interface function, passing an
+ *            operation and the private driver state.  The driver will call 
+ *            globus_xio_driver_pass_close() passing it the operation.
  *            When the close callback is received the driver calls
  *            globus_xio_driver_finished_close() passing it the
- *            operation received in the close callback and the driver_handle
- *            received in the interface function.  At this point the 
- *            driver_handle is no longer valid.
+ *            operation. At this point, the driver typically frees its
+ *            private data, as the handle is no longer valid.
  *            </dd>
  *    </dl>
- */
-/**
- *   @addtogroup globus_xio_driver
  *
- *   @section advanced_driver_programming Advanced Driver Programming
+ *  @section globus_xio_driver_attributes Globus XIO Attributes
+ *  Attributes provide a way to pass additional metadata to driver operations.
+ *  Most Globus XIO API functions include an attribute parameter. Each driver
+ *  implements its own set of attribute handling function. To implement
+ *  attributes, a driver must provide functions of these types:
+ *
+ *  - globus_xio_driver_attr_init_t
+ *  - globus_xio_driver_attr_copy_t
+ *  - globus_xio_driver_attr_cntl_t
+ *  - globus_xio_driver_attr_destroy_t
+ *
+ *  Pointers to these functions are associated with the driver by calling
+ *  globus_xio_driver_set_attr
+ *
+ *   @section globus_xio_advanced_driver_programming Advanced Driver Programming
  *   The typical driver implementation is describe above.  However globus_xio
- *   allows driver authors to do more advanced things.  Some of these things
- *   will be explored here.
+ *   allows driver authors to do more advanced things such as initiating
+ *   operations on their own.
  *
  *   <dl>
- *   <dt><tt>Read Ahead</tt></dt>
- *   <dd>Once a driver_handle is open a driver can spawn operation structures
- *   from it.  This gives the driver the ability to request io from the driver
- *   stack before it receives a call to its own interface io interface
- *   function.  So if a driver wishes to read ahead it does the following:
+ *   <dt>Read Ahead</dt>
+ *   <dd>Once a handle is open, a driver can create operation structures
+ *   from it.  A driver can then request I/O from the driver
+ *   stack before it receives a call to its own I/O interface
+ *   functions, implementing read-ahead functionality. This can be done
+ *   as follows:
  *
- *   \li it creats an operation by calling globus_xio_driver_create_operation()
- *   and passing it the driver_handle it is interested in using.
- *
- *   \li call globus_xio_driver_read() using this operations.  When the read
- *   callback is received the driver may call finished_operation() on the
- *   op it receives (this ultimately results in very little, since this 
- *   operation was started by this driver, but it is good practice and will
- *   free up resources that would otherwise leak).
- *
- *   \li Now when the user finally does receive a read interface call from
- *   globus_xio it can immediately finish it using the operation it just
- *   received as a parameter and updating the iovec structure to represent 
- *   the read that already occurred.
- *   </dd>
- *
- *   <dt><tt>Pre-Opening Handles</tt></dt>
- *   <dd>
- *   Once the driver has received a globus_xio_driver_stack_t it can 
- *   open a driver_handle.  The globus_xio_driver_stack_t comes in the
- *   call to the interface function globus_xio_server/client_init_t().
- *   The driver uses this structure in a call to globus_xio_driver_open().
- *   When this functionality completes the driver has an initialized 
- *   driver_handle and can use it to create operations as described above.
- *   The driver can now hang onto this driver_handle until it receives an open
- *   interface function call.  At which time it can call 
- *   globus_xio_driver_finished_open() passing in the driver_handle and thereby
- *   glueing the pre opened driver_handle with the requested globus_xio 
- *   operation.
+ *   <ol>
+ *   <li>Obtain a driver handle from the open operation by calling
+ *   globus_xio_operation_get_driver_handle()</li>
+ *   <li>Create an operation by calling globus_xio_driver_operation_create()
+ *   and pass it the driver_handle from the previous step</li>
+ *   <li>call globus_xio_driver_pass_read() using this operation.  When the
+ *   read callback is received, the driver saves the returned data in
+ *   its private data and then calls globus_xio_driver_finished_read() on the
+ *   operation.</li>
+ *   <li>When XIO calls the driver's read interface function,
+ *   the driver can immediately call globus_xio_driver_finished_read()
+ *   function after updating the iovec structure with the data it had
+ *   previously read.</li>
+ *   </ol>
  *   </dd>
  *   </dl>
  */
-/**
- *  @defgroup globus_xio_driver_programming XIO Driver Programming
- *  @ingroup globus_xio_driver
- *  The set of interface functions that the driver author must implement 
- *  to create a driver and the functions to assist in the creation.
- *
- *  Driver attribute functions
- *
- *  If the driver wishes to provide driver specific attributes to the
- *  user it must implement the following functions:
- *
- *  globus_xio_driver_attr_init_t
- *  globus_xio_driver_attr_copy_t
- *  globus_xio_driver_attr_cntl_t
- *  globus_xio_driver_attr_destroy_t
- */
 
 /*******************************************************************
  *                        callbacks
  ******************************************************************/
 /**
- *  callback interface
- *  @ingroup globus_xio_driver_programming
- *
+ * @brief Open and Close Callback Signature
+ * @ingroup globus_xio_driver
+ * @details
  *  This is the function signature of callbacks for the 
- *  globus_xio_driver_open/close().
+ *  globus_xio_pass_open() and globus_xio_pass_close() functions.
  *
  * @param op
- *         The operation structure associated with the open or the
- *         close requested operation.  The driver should call the 
- *         appropriate finished operation to clean up this structure.
- *
+ *     The operation structure associated with the open or the
+ *     close requested operation.  The driver should call the 
+ *     appropriate finished operation to clean up this structure.
  * @param result
- *         The result of the requested data operation
- *  
+ *     The result of the requested data operation
  * @param user_arg
- *         The user pointer that is threaded through to the callback.
+ *     The user pointer that is threaded through to the callback.
  */
 typedef void
 (*globus_xio_driver_callback_t)(
@@ -292,7 +277,7 @@ typedef void
 
 /**
  *  Data Callback interface
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This is the function signature of read and write operation 
  *  callbacks.  
@@ -339,7 +324,7 @@ globus_xio_driver_data_descriptor_cntl(
     ...);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Touch driver specific information in a handle object.
  *
  *  pass the driver to control a specific driver
@@ -355,13 +340,13 @@ globus_xio_driver_handle_cntl(
     ...);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 typedef globus_result_t
 (*globus_xio_driver_get_driver_t)(
     globus_xio_driver_t *               out_driver);
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Create a driver specific attribute.
  *
  *  The driver should implement this function to create a driver 
@@ -373,7 +358,7 @@ typedef globus_result_t
     void **                             out_driver_attr);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Copy a driver attr.
  *
  *  When this function is called the driver will create a copy of the attr 
@@ -385,7 +370,7 @@ typedef globus_result_t
     void *                              src);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Destroy the driver attr.
  *
  *  Clean up all resources associate with the attr. 
@@ -396,7 +381,7 @@ typedef globus_result_t
     void *                              driver_attr);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  get or set information in an attr.
  *
  *  The cmd parameter determines what functionality the user is requesting.
@@ -421,30 +406,29 @@ typedef globus_result_t
     va_list                             ap);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Initialize a server object
  *
  *  The driver developer should implement this function if their driver
- *  handles server operations (passive opens).  In the tcp driver this 
- *  function should create a listener.
+ *  handles server operations (passive opens).  In the TCP driver this 
+ *  function creates a TCP socket and calls listen() on it. Unlike all
+ *  other XIO driver implementation functions, the globus_xio_server_create()
+ *  function begins at the bottom (transport driver) of the stack, and
+ *  the globus_xio_driver_pass_server_init() acts like a finished() operation.
  *
- *  @param op
- *         An op which should be passed to globus_xio_driver_server_created.
- *         Note, that unlike most operations, the server is created from
- *         the bottom of the stack to the top.
- * 
- *  @param contact_info
- *         This the contact info for the stack below this driver.
- *         (entries will always be NULL for the transport driver)
- *         
  *  @param driver_attr
- *         A server attr if the user specified any driver specific 
- *         attributes.  This may be NULL.
+ *         A driver-specific attribute. This may be NULL.
+ *  @param contact_info
+ *         Contact information from the stack below this driver.
+ *         This is NULL for the transport driver.
+ *  @param op
+ *         An op which must be passed to globus_xio_driver_pass_server_init()
+ *         to pass the server contact information up the XIO driver stack.
  * 
  *  @return
  *         Returning GLOBUS_SUCCESS for this means that 
- *  `      globus_xio_driver_pass_server_init returned success and
- *         the driver's server was successfully initialized.
+ *         globus_xio_driver_pass_server_init returned success and
+ *         the driver's server-specific data is initialized.
  */
 typedef globus_result_t
 (*globus_xio_driver_server_init_t)(
@@ -473,8 +457,9 @@ typedef globus_result_t
  *         it is acceptable to 'steal' pointers from the received contact_info.
  * 
  *  @param driver_server
- *         The driver's server handle.  Future calls to server accept or server
- *         cntl will be passed this value.
+ *         The driver's server-specific state.  Future calls to the
+ *         server_accept, server_cntl, and server_destroy will be passed this
+ *         value.
  */
 globus_result_t
 globus_xio_driver_pass_server_init(
@@ -483,7 +468,7 @@ globus_xio_driver_pass_server_init(
     void *                              driver_server);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  destroy a server.
  *
  *  When this function is called the driver should free up all resources
@@ -501,7 +486,7 @@ typedef globus_result_t
     void *                              driver_server);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Accept a server connection
  *
  *  The driver developer should implement this function if their driver 
@@ -532,7 +517,7 @@ globus_xio_driver_pass_accept(
     void *                              in_user_arg);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 typedef void
 (*globus_xio_driver_cancel_callback_t)(
@@ -541,7 +526,7 @@ typedef void
     globus_xio_error_type_t             reason);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Driver API finished accept
  *
  *  This function should be called to signal globus_xio that it has 
@@ -566,7 +551,7 @@ globus_xio_driver_finished_accept(
     globus_result_t                     result);
     
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  Query a server for information.
  *
  *  This function allows a user to request information from a driver
@@ -590,7 +575,7 @@ typedef globus_result_t
 
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 typedef globus_result_t
 (*globus_xio_driver_link_cntl_t)(
@@ -599,7 +584,7 @@ typedef globus_result_t
     va_list                             ap);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *  destroy a link
  *
  *  The driver should clean up all resources associated with the link
@@ -618,11 +603,16 @@ typedef globus_result_t
  *********************************************************************/
 
 /**
- *  Open a handle
- *  @ingroup globus_xio_driver_programming
- *
- *  This is called when a user requests to open a handle.
+ * @brief Open a handle
+ * @ingroup globus_xio_driver
+ * @details
+ * This is called when a user opens a handle.
  *
+ *  @param contact_info
+ *         Contains information about the requested resource.  Its members
+ *         may all be null (especially when link is not null).  XIO will
+ *         destroy this contact info upon return from the interface function
+ *    
  *  @param driver_link
  *         Comes from server accept.  Used to link an accepted connection to
  *         an xio handle.  XIO will destroy this object upon the return of
@@ -630,19 +620,13 @@ typedef globus_result_t
  *
  *  @param driver_attr
  *         A attribute describing how to open.  This points to a piece of 
- *         memory created by the globus_xio_driver_driver_attr_init_t
+ *         memory created by the globus_xio_driver_attr_init_t
  *         interface function.
  *
- *  @param contact_info
- *         Contains information about the requested resource.  Its members
- *         may all be null (especially when link is not null).  XIO will
- *         destroy this contact info upon return from the interface function
- *    
  *  @param op
- *         The requested operation.  When the driver is finished opening
- *         the handle it uses this structure to signal globus_xio that it
- *         has completed the operation requested.  It does this by calling
- *         globus_xio_driver_finished_open()
+ *         The open operation.  When the driver is finished opening
+ *         the handle, it passes this to globus_xio_driver_finished_open()
+ *         to return status up the driver stack to the application.
  *         
  */
 typedef globus_result_t
@@ -653,8 +637,8 @@ typedef globus_result_t
     globus_xio_operation_t              op);
 
 /**
- *  @ingroup globus_xio_driver_programming
- *  transport open
+ * @ingroup globus_xio_driver
+ * @copydoc globus_xio_driver_transform_open_t
  */
 typedef globus_result_t
 (*globus_xio_driver_transport_open_t)(
@@ -665,7 +649,7 @@ typedef globus_result_t
 
 /**
  *  Driver API Open
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function will pass an open request down the driver stack.
  *  Upon completion of the open operation globus_xio will call the @a cb 
@@ -700,7 +684,7 @@ globus_xio_driver_pass_open(
 
 /**
  *  Driver API finished open
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function should be called to signal globus_xio that it has 
  *  completed the open operation requested of it.  It will free up 
@@ -728,7 +712,7 @@ globus_xio_driver_finished_open(
  *********************************************************************/
 /**
  *  Driver API Create Operation
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function will create an operation from an initialized handle
  *  This operation can then be used for io operations related to the
@@ -750,14 +734,14 @@ globus_xio_driver_operation_create(
     globus_xio_driver_handle_t          handle);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 void
 globus_xio_driver_operation_destroy(
     globus_xio_operation_t              operation);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_operation_cancel(
@@ -766,7 +750,7 @@ globus_xio_driver_operation_cancel(
 
 /**
  *  Is Operation blocking.
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  If the operation is blocking the driver developer may be able to make
  *  certain optimizations.  The function returns true if the given operation
@@ -777,7 +761,7 @@ globus_xio_driver_operation_is_blocking(
     globus_xio_operation_t              operation);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  * 
  *  this call *must* return an GLOBUS_XIO_ERROR_COMMAND error for unsupported
  *  command numbers.   (use GlobusXIOErrorInvalidCommand(cmd))
@@ -803,25 +787,23 @@ globus_xio_driver_merge_handle(
  *                          Close
  *********************************************************************/
 /**
- *  Close an open handle
- *  @ingroup globus_xio_driver_programming
+ * @brief Close a handle
+ * @ingroup globus_xio_driver
+ * @details
+ * This closes a handle. Driver implementations should pass the close to the
+ * other drivers in the stack by calling globus_xio_pass_close().
  *
- *  This is called when a user requests to close a handle.  The driver 
- *  implementor should clean up all resources connected to there driver handle
- *  when this function is called.
- *
- *  @param driver_specific_handle
- *         The driver handle to be closed.
- *
- *  @param driver_attr
- *         A driver specific attr which may be used to alter how a close
- *         is performed (e.g. caching drivers)
- * 
- *  @param op
- *         The requested operation.  When the driver is finished closing
- *         the handle it uses this structure to signal globus_xio that it
- *         has completed the operation requested.  It does this by calling
- *         globus_xio_driver_finished_close()
+ * In the close callback, the driver should clean up the data
+ * associated with driver_handle.
+ * @param driver_specific_handle
+ *     The driver handle to be closed.
+ * @param driver_attr
+ *     A driver specific attr which may be used to alter how a close
+ *     is performed (e.g. caching drivers)
+ * @param op
+ *     The open operation.  When the driver is finished opening
+ *     the handle, it passes this to globus_xio_driver_finished_close()
+ *     to return status up the driver stack to the application.
  */
 typedef globus_result_t
 (*globus_xio_driver_close_t)(
@@ -830,32 +812,30 @@ typedef globus_result_t
     globus_xio_operation_t              op);
 
 /**
- *  Driver API Close
- *  @ingroup globus_xio_driver_programming
+ * @brief Pass the close operation down the driver stack
+ * @ingroup globus_xio_driver
+ * @details
+ * This function will pass a close request down the driver stack.  Upon
+ * completion of the close operation globus_xio will call the function
+ * pointed to by the cb parameter.
  *
- *  This function will pass a close request down the driver stack.  Upon
- *  completion of the close operation globus_xio will call the function
- *  pointed to by the cb argument.
- *
- *  @param op
- *         The operation to pass along the driver stack for closing.
- *
- *  @param cb
- *         A pointer to the function to be called once all drivers lower
- *         in the stack have closed.
- *
- *  @param user_arg
- *         A user pointer that will be threaded through to the callback.
+ * @param op
+ *     The close operation.
+ * @param cb
+ *     A pointer to the function to be called once all drivers lower
+ *     in the stack have closed.
+ * @param callback_arg
+ *     A pointer that will be passed to the callback.
  */
 globus_result_t
 globus_xio_driver_pass_close(
     globus_xio_operation_t              op,
     globus_xio_driver_callback_t        cb,
-    void *                              user_arg);
+    void *                              callback_arg);
 
 /**
  *  Driver API finished_close
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  The driver calls this function after completing a close operation
  *  on a driver_handle.  Once this function returns the driver_handle is 
@@ -878,7 +858,7 @@ globus_xio_driver_finished_close(
  *********************************************************************/
 /**
  *  Read data from an open handle.
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function is called when the user requests to read data from 
  *  a handle.  The driver author shall implement all code needed to for
@@ -908,7 +888,7 @@ typedef globus_result_t
 
 /**
  *  Driver read
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function passes a read operation down the driver stack.  After
  *  this function is called the op structure is no longer valid.  However
@@ -948,7 +928,7 @@ globus_xio_driver_pass_read(
 
 /**
  *  Finished Read
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function is called to signal globus_xio that the requested
  *  read operation has been completed.
@@ -971,7 +951,7 @@ globus_xio_driver_finished_read(
 
 /**
  * EOF state manipulation
- * @ingroup globus_xio_driver_programming
+ * @ingroup globus_xio_driver
  * 
  * This function is used by drivers that allow multiple outstanding reads at
  * a time.  It can only be called on behalf of a read operation (while in the
@@ -999,7 +979,7 @@ globus_xio_driver_set_eof_received(
 
 /**
  * EOF state checking
- * @ingroup globus_xio_driver_programming
+ * @ingroup globus_xio_driver
  * 
  * This function is used by drivers that allow multiple outstanding reads at
  * a time.  It can only be called on behalf of a read operation (while in the
@@ -1028,7 +1008,7 @@ globus_xio_driver_eof_received(
  *********************************************************************/
 /**
  *  Write data from an open handle.
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function is called when the user requests to write data to
  *  a handle.  The driver author shall implement all code needed to for
@@ -1058,7 +1038,7 @@ typedef globus_result_t
 
 /**
  *  Driver write
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function passes a write operation down the driver stack.  After
  *  this function is called the op structure is no longer valid.  However
@@ -1099,7 +1079,7 @@ globus_xio_driver_pass_write(
 
 /**
  *  Finished Write
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function is called to signal globus_xio that the requested
  *  write operation has been completed.
@@ -1123,7 +1103,7 @@ globus_xio_driver_finished_write(
 /**
  *  Finishes an operation and merge two op structures.
  *  (XXX not implemented yet)
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  *
  *  This function will join to operations together and signal globus_xio
  *  that it has completed.  This is an advanced function.  Most drivers will
@@ -1217,16 +1197,13 @@ globus_xio_driver_init(
     void *                              user_data);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_get_user_data(
     globus_xio_driver_t                 in_driver,
     void **                             out_user_data);
 
-/**
- *  @ingroup globus_xio_driver_programming
- */
 globus_result_t
 globus_xio_operation_attr_cntl(
     globus_xio_operation_t              op,
@@ -1234,14 +1211,14 @@ globus_xio_operation_attr_cntl(
     ...);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_destroy(
     globus_xio_driver_t                 driver);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_set_transport(
@@ -1253,7 +1230,7 @@ globus_xio_driver_set_transport(
     globus_xio_driver_handle_cntl_t     handle_cntl_func);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_set_transform(
@@ -1266,7 +1243,7 @@ globus_xio_driver_set_transform(
     globus_xio_driver_push_driver_t     push_driver_func);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_set_server(
@@ -1278,8 +1255,12 @@ globus_xio_driver_set_server(
     globus_xio_driver_link_cntl_t       link_cntl_func,
     globus_xio_driver_link_destroy_t    link_destroy_func);
 
+globus_result_t
+globus_xio_driver_set_server_pre_init(
+    globus_xio_driver_t                 driver,
+    globus_xio_driver_server_init_t     server_pre_init_func);
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 globus_result_t
 globus_xio_driver_set_attr(
@@ -1331,6 +1312,10 @@ globus_xio_driver_t
 globus_xio_operation_get_user_driver(
     globus_xio_operation_t              op);
 
+globus_xio_driver_t
+globus_xio_operation_get_transport_user_driver(
+    globus_xio_operation_t              op);
+
 /* this returns the handle to the drivers below you */
 globus_xio_driver_handle_t
 globus_xio_operation_get_driver_handle(
@@ -1356,7 +1341,7 @@ globus_xio_operation_copy_stack(
 
 /* STRING PARSING STUFF */
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 typedef globus_result_t
 (*globus_xio_string_cntl_parse_func_t)(
@@ -1367,11 +1352,11 @@ typedef globus_result_t
     globus_xio_driver_attr_cntl_t       cntl_func);
 
 /**
- *  @ingroup globus_xio_driver_programming
+ *  @ingroup globus_xio_driver
  */
 typedef struct globus_xio_string_cntl_table_s
 {
-    char *                              key;
+    const char *                        key;
     int                                 cmd;
     globus_xio_string_cntl_parse_func_t parse_func;
 } globus_xio_string_cntl_table_t;
@@ -1402,7 +1387,7 @@ typedef struct globus_xio_string_cntl_table_s
  *  all at runtime with no object module references.  For example, a TCP driver
  *  can be loaded with the string: tcp, and the options can be set with:
  *
- *  port=50668\#keepalive=yes\#nodelay=N
+ *  port=50668;keepalive=yes;nodelay=N
  *
  *  this would set the port to 50668, keepalive to true and nodelay to false.
  *  The particular string definition is defined by the tcp driver by properly 
@@ -1424,17 +1409,24 @@ typedef struct globus_xio_string_cntl_table_s
  *  - @ref globus_xio_string_cntl_int
  *  - @ref globus_xio_string_cntl_string
  *  - @ref globus_xio_string_cntl_int_int
- *
  */
 /**
- *  @ingroup string_globus_xio_driver_programming
- *
+ * Set the string table for attr cntl
+ * @ingroup string_globus_xio_driver_programming
  */
 globus_result_t
 globus_xio_driver_string_cntl_set_table(
     globus_xio_driver_t                 driver,
     globus_xio_string_cntl_table_t *   table);
 
+/**
+ * Set the string table for handle cntl
+ * @ingroup string_globus_xio_driver_programming
+ */
+globus_result_t
+globus_xio_driver_handle_string_cntl_set_table(
+    globus_xio_driver_t                driver,
+    globus_xio_string_cntl_table_t *   table);
 
 /**
  *  @ingroup string_globus_xio_driver_programming
