@@ -206,7 +206,9 @@ int globus_utp_init(unsigned numTimers, int mode)
 #endif
 
 	if (globus_libc_gethostname(hostnameBuff, MAXHOSTNAMELEN)) {
-	    memset(hostnameBuff, '\0', MAXHOSTNAMELEN);
+	    /* globus_utp_warn("globus_utp_init: gethostname() failed; system error is "
+		         "\"%s\"", sys_errlist[errno]); */
+		return 1;
 	}
 	globus_utp_set_attribute("%s", "hostname", hostnameBuff);
 
